#ifndef CHESSY_MOVEGENERATION_H
#define CHESSY_MOVEGENERATION_H

#include "BitBoard.h"

namespace chessy
{
    struct Move;

    class PositionSet;

    class MoveGenerator;
}


struct chessy::Move
{
    Index origin;
    Index destination;

    Move(void)          = default;
    Move(const Move&)   = default;
    ~Move(void)         = default;

    inline Move(Index origin, Index destination) :
        origin{origin}, destination{destination} {}
};


class chessy::PositionSet
{
    Bitboard bitboard;

public:
    struct PositionSetIterator
    {
        Bitboard bitboard;

        inline Index operator *(void) const
        {
            //if (bitboard == Bitboard(0)) return -1;
            return Bitboard::trailingZeroes(bitboard.bits);
        }

        inline void operator ++(void)
        {
            // remove least significant one-bit
            bitboard.bits &= bitboard.bits - 1;
        }

        inline bool operator !=(const PositionSetIterator& psi) const
        {
            return bitboard != psi.bitboard;
        }
    };

    inline PositionSetIterator begin(void) const
    {
        return PositionSetIterator {bitboard};
    }
    inline PositionSetIterator end(void) const
    {
        return PositionSetIterator {0};
    }
};


class chessy::MoveGenerator
{
    struct MoveIterator
    {
        PositionSet::PositionSetIterator pawnPushes;
        inline Move operator *(void) const
        {
            Index pp = *pawnPushes;
            return Move(pp - 8, pp);
        }

        inline void operator ++(void)
        {
            ++pawnPushes;
        }

        inline bool operator !=(const MoveIterator& psi) const
        {
            pawnPushes != PositionSet::PositionSetIterator {0};
        }
    };
public:
    MoveGenerator(void);
    //PawnPushProducer generatePawnPushes(void) const;
};



#endif // CHESSY_MOVEGENERATION_H
